#!/usr/bin/python3

#
# Start up the "widgets" file system demo
#

import os
import sys
import json
import getopt
from string import Template

assert sys.version_info >= (3, 7)

script_path = os.path.dirname(os.path.realpath(__file__))
biz_demo_path = script_path + '/../../../db-adaptor/example/biz'

# biz-demo will be in a different place when following the public demo instructions
if not os.path.isdir(biz_demo_path):
    biz_demo_path = script_path + '/../../biz-demo'
if not os.path.isdir(biz_demo_path):
    print('script_path=' + script_path)
    print('biz_demo_path=' + biz_demo_path)
    print('Cannot find biz demo directory: Update script variable to point to the biz demo installation directory: biz_demo_path')
    exit(2)
biz_demo_scripts_path = biz_demo_path + "/scripts"


# add the biz demo scripts path so those scripts can be used
sys.path.insert(1, biz_demo_scripts_path)
import adaptors
import domains
import zones
from functions import exec_command, list_to_dict, k8s_apply_stdin, k8s_drain

def usage():
    print("Usage: start.py [-d | -l] [-i] -s (source URI) -x (destination URI)")
    print("-d = AWS Demo")
    print("-l = Local docker")
    print("-s = source URI for local mode (ie file:///d:/source or s3://fs-demo-source/)")
    print("-x = destination URI for local mode (ie file:///d:/dest or s3://fs-demo-dest/)")
    print("-i = When running locally (-l), use the local younite-fs-adaptor Docker image (default is to pull the latest from the repo)")
    exit(1)

try:
    opts, _ = getopt.getopt(sys.argv[1:], "dlis:x:")
except getopt.GetoptError as err:
    print("ERROR: " + err.msg + "\n")
    usage()

global_file = script_path + "/local_global.json"

namespace = "default"
image_tag = "latest"
aws_region = "us-west-2"

aws_access_key_id = ""
aws_secret_access_key = ""

message_bus_url = "tcp://younite-mb:61616"
demo = False
local = False
zones_only = False
domains_only = False

repo_fs_adaptor_image = "160909222919.dkr.ecr.us-west-2.amazonaws.com/younite/younite-fs-adaptor:latest"
local_fs_adaptor_image = "younite/younite-fs-adaptor"
fs_adaptor_image = repo_fs_adaptor_image


# concurrency at this time needs to be 1 to ensure
# files aren't processed out of order (TODO ?)
concurrency = 1
log_level = "INFO"

source_uri = None
dest_uri = None

for o, v in opts:
    if o == "-d":
        demo = True
        global_file = script_path + "/aws_global.json"
    elif o == "-l":
        local = True
    elif o == "-s":
        source_uri = v
    elif o == "-x":
        dest_uri = v
    elif o == "-i":
        fs_adaptor_image = local_fs_adaptor_image

if not (demo ^ local):
    usage()


if local and (source_uri is None or dest_uri is None):
    usage()


# stop and remove running containers
if local:
    adaptors.stop_adaptors_local(local_fs_adaptor_image)
    adaptors.stop_adaptors_local(repo_fs_adaptor_image)
else:
    k8s_drain(["source-fs-adaptor", "destination-fs-adaptor"], namespace)

# use the same zones as the biz demo
zones.create_zones(global_file)

# create the widgets domain
domains.create_domain(script_path + "/domains/widgets.json", global_file)

# create the source adaptor
source_adaptor = adaptors.create_adaptor(script_path + "/adaptors/source-adaptor.json", global_file,
    script_path + "/adaptors/source-template.json", None, '23d5cab5-79c9-47f0-8e29-70915bf70d04')

# create the destination adaptor
destination_adaptor = adaptors.create_adaptor(script_path + "/adaptors/destination-adaptor.json", global_file,
    script_path + "/adaptors/destination-template.json", None, '23d5cab5-79c9-47f0-8e29-70915bf70d04')

#
# Local
#
if local:
    mounts = []
    # for local files, use volume mounts and change the URIs to reflect that
    if source_uri.startswith("file:///"):
        uindex = 8 if source_uri[9] == ':' else 7   # accommodate windows and unix paths
        mounts = mounts + ["-v", source_uri[uindex:] + ":/monitor/sources"]
        source_uri = "file:///monitor/sources"

    if dest_uri.startswith("file:///"):
        uindex = 8 if dest_uri[9] == ':' else 7    # accommodate windows and unix paths
        mounts = mounts + ["-v", dest_uri[uindex:] + ":/monitor/destinations"]
        dest_uri = "file:///monitor/destinations"

    # When running local, we handle separate AWS accounts for ECR images and S3 buckets
    # - AWS_ is for ECR access
    # - S3_AWS_ is for S3 bucket access but if none is defined then AWS_  is used as default for both
    #
    if source_uri.startswith("s3://") or dest_uri.startswith("s3://"):
        aws_access_key_id = os.getenv("S3_AWS_ACCESS_KEY_ID")
        if aws_access_key_id is None:
            aws_access_key_id = os.getenv("AWS_ACCESS_KEY_ID")
        aws_secret_access_key = os.getenv("S3_AWS_SECRET_ACCESS_KEY")
        if aws_secret_access_key is None:
            aws_secret_access_key = os.getenv("AWS_SECRET_ACCESS_KEY")

        if aws_access_key_id is None or aws_secret_access_key is None:
            print("ERROR, AWS_ACCESS_KEY_ID and AWS_SECRET_ACCESS_KEY environment variables not set!")
            exit(1)

    if destination_adaptor:
        cmd = ["docker", "run", "--name", "destination-fs-adaptor", "--network", "younite", "--expose", "8001",
               "-e", "ADAPTOR_UUID=" + destination_adaptor["uuid"],
               "-e", "CLIENT_ID=" + destination_adaptor["clientId"],
               "-e", "CLIENT_SECRET=" + destination_adaptor["clientSecret"],
               "-e", "MESSAGE_BUS_URL=" + message_bus_url,
               "-e", "AWS_ACCESS_KEY_ID=" + aws_access_key_id,
               "-e", "AWS_SECRET_ACCESS_KEY=" + aws_secret_access_key,
               "-e", "AWS_REGION=" + aws_region,
               "-e", "LOG_LEVEL=" + log_level,
               "-e", "SOURCE_URI=" + source_uri,
               "-e", "DESTINATION_URI=" + dest_uri,
               "-e", "CONCURRENCY=" + str(concurrency)] + mounts + ["-d", fs_adaptor_image]

        exec_command(cmd)

    if source_adaptor:
        cmd = ["docker", "run", "--name", "source-fs-adaptor", "--network", "younite", "--expose", "8001",
               "-e", "ADAPTOR_UUID=" + source_adaptor["uuid"],
               "-e", "CLIENT_ID=" + source_adaptor["clientId"],
               "-e", "CLIENT_SECRET=" + source_adaptor["clientSecret"],
               "-e", "MESSAGE_BUS_URL=" + message_bus_url,
               "-e", "AWS_ACCESS_KEY_ID=" + aws_access_key_id,
               "-e", "AWS_SECRET_ACCESS_KEY=" + aws_secret_access_key,
               "-e", "AWS_REGION=" + aws_region,
               "-e", "LOG_LEVEL=" + log_level,
               "-e", "SOURCE_URI=" + source_uri,
               "-e", "CONCURRENCY=" + str(concurrency)] + mounts + ["-d", fs_adaptor_image]

        exec_command(cmd)

#
# AWS demo
#
else:

    source_uri = "s3://fs-demo-source/"
    dest_uri = "s3://fs-demo-destination/"

    # apply secrets
    aws_access_key_id = os.getenv("AWS_ACCESS_KEY_ID")
    aws_secret_access_key = os.getenv("AWS_SECRET_ACCESS_KEY")

    if aws_access_key_id is None or aws_secret_access_key is None:
        print("ERROR, AWS_ACCESS_KEY_ID and AWS_SECRET_ACCESS_KEY environment variables not set!")
        exit(1)

    f = open(script_path + "/aws-s3-secrets.yml")
    try:
        secrets_template = f.read()
    finally:
        f.close()

    yml = Template(Template(secrets_template).safe_substitute(locals())).safe_substitute(globals())

    k8s_apply_stdin(yml, namespace)

    f = open(script_path + "/fs-adaptor.yml")
    try:
        adaptor_template = f.read()
    finally:
        f.close()

    if source_adaptor:
        container_name = "source-fs-adaptor"
        adaptor_uuid = source_adaptor["uuid"]
        client_id = source_adaptor["clientId"]
        client_secret = source_adaptor["clientSecret"]
        java_opts = ""

        yml = Template(Template(adaptor_template).safe_substitute(locals())).safe_substitute(globals())
        k8s_apply_stdin(yml, namespace)

    if destination_adaptor:
        container_name = "destination-fs-adaptor"
        adaptor_uuid = destination_adaptor["uuid"]
        client_id = destination_adaptor["clientId"]
        client_secret = destination_adaptor["clientSecret"]
        java_opts = ""

        yml = Template(Template(adaptor_template).safe_substitute(locals())).safe_substitute(globals())
        k8s_apply_stdin(yml, namespace)
